@@ -62,8 +62,8 @@ public:
 
     virtual void OnIteration(int type, void *data, uint64_t iteration) = 0;
 
-    form* forms;
-    int64_t iterations = 0;
+    std::unique_ptr<form[]> forms;
+    std::atomic<int64_t> iterations{0};
     integer D;
     integer L;
     PulmarkReducer* reducer;
@@ -72,7 +72,7 @@ public:
 
 class OneWesolowskiCallback: public WesolowskiCallback {
   public:
-    OneWesolowskiCallback(integer& D, uint64_t wanted_iter) : WesolowskiCallback(D) {
+    OneWesolowskiCallback(integer& D, form& f, uint64_t wanted_iter) : WesolowskiCallback(D) {
         uint32_t k, l;
         this->wanted_iter = wanted_iter;
         if (wanted_iter >= (1 << 16)) {
@@ -83,15 +83,10 @@ class OneWesolowskiCallback: public WesolowskiCallback {
         }
         kl = k * l;
         uint64_t space_needed = wanted_iter / (k * l) + 100;
-        forms = (form*) calloc(space_needed, sizeof(form));
-        form f = form::generator(D);
+        forms.reset(new form[space_needed]);
         forms[0] = f;
     }
 
-    ~OneWesolowskiCallback() {
-        free(forms);
-    }
-
     void OnIteration(int type, void *data, uint64_t iteration) {
         iteration++;
         if (iteration > wanted_iter) 
@@ -114,19 +109,14 @@ class OneWesolowskiCallback: public WesolowskiCallback {
 
 class TwoWesolowskiCallback: public WesolowskiCallback {
   public:
-    TwoWesolowskiCallback(integer& D) : WesolowskiCallback(D) {
+    TwoWesolowskiCallback(integer& D, form f) : WesolowskiCallback(D) {
         int space_needed = kSwitchIters / 10 + (kMaxItersAllowed - kSwitchIters) / 100;
-        forms = (form*) calloc(space_needed, sizeof(form));
-        form f = form::generator(D);
+        forms.reset(new form[space_needed]);
         forms[0] = f;
         kl = 10;
         switch_iters = -1;
     }
 
-    ~TwoWesolowskiCallback() {
-        free(forms);
-    }
-
     void IncreaseConstants(uint64_t num_iters) {
         kl = 100;
         switch_iters = num_iters;
@@ -166,8 +156,7 @@ class TwoWesolowskiCallback: public WesolowskiCallback {
 
 class FastAlgorithmCallback : public WesolowskiCallback {
   public:
-    FastAlgorithmCallback(int segments, integer& D, bool multi_proc_machine) : WesolowskiCallback(D) {
-        form f = form::generator(D);
+    FastAlgorithmCallback(int segments, integer& D, form f, bool multi_proc_machine) : WesolowskiCallback(D) {
         buckets_begin.push_back(0);
         buckets_begin.push_back(bucket_size1 * window_size);
         this->segments = segments;
@@ -176,20 +165,15 @@ class FastAlgorithmCallback : public WesolowskiCallback {
             buckets_begin.push_back(buckets_begin[buckets_begin.size() - 1] + bucket_size2 * window_size);
         }
         int space_needed = window_size * (bucket_size1 + bucket_size2 * (segments - 1));
-        forms = (form*) calloc(space_needed, sizeof(form));
-        checkpoints = (form*) calloc((1 << 18), sizeof(form));
+        forms.reset(new form[space_needed]);
+        checkpoints.reset(new form[1 << 18]);
 
-        y_ret = form::generator(D);
+        y_ret = f;
         for (int i = 0; i < segments; i++)
             forms[buckets_begin[i]] = f;
         checkpoints[0] = f;
     }
 
-    ~FastAlgorithmCallback() {
-        free(checkpoints);
-        free(forms);
-    }
-
     int GetPosition(uint64_t exponent, int bucket) {
         uint64_t power_2 = 1LL << (16 + 2 * bucket);
         int position = buckets_begin[bucket];
@@ -234,7 +218,7 @@ class FastAlgorithmCallback : public WesolowskiCallback {
     }
 
     std::vector<int> buckets_begin;
-    form* checkpoints;
+    std::unique_ptr<form[]> checkpoints;
     form y_ret;
     int segments;
     // The intermediate values size of a 2^16 segment.
