@@ -168,6 +168,11 @@ struct integer {
         mpz_import(impl, data.size(), -1, 8, 0, 0, &data[0]);
     }
 
+    integer(const uint8_t *bytes, size_t size) {
+        mpz_init(impl);
+        mpz_import(impl, size, 1, 1, 1, 0, bytes);
+    }
+
     //lsb first
     vector<uint64> to_vector() const {
         vector<uint64> res;
@@ -180,6 +185,13 @@ struct integer {
         return res;
     }
 
+    vector<uint8_t> to_bytes() const {
+        vector<uint8_t> res((mpz_sizeinbase(impl, 2) + 7) / 8);
+
+        mpz_export(res.data(), NULL, 1, 1, 0, 0, impl);
+        return res;
+    }
+
     integer& operator=(const integer& t) {
         mpz_set(impl, t.impl);
         return *this;
