@@ -24,28 +24,28 @@ int disc_int_size;
 
 void WriteProof(uint64_t iteration, Proof& result, tcp::socket& sock) {
     // Writes the number of iterations
-    std::vector<unsigned char> bytes = ConvertIntegerToBytes(integer(iteration), 8);
+    uint8_t int_bytes[8];
+    std::vector<uint8_t> bytes;
+    Int64ToBytes(int_bytes, iteration);
+    VectorAppendArray(bytes, int_bytes, sizeof(int_bytes));
 
     // Writes the y, with prepended size
-    std::vector<unsigned char> y_size = ConvertIntegerToBytes(integer(result.y.size()), 8);
-    bytes.insert(bytes.end(), y_size.begin(), y_size.end());
-    bytes.insert(bytes.end(), result.y.begin(), result.y.end());
+    Int64ToBytes(int_bytes, result.y.size());
+    VectorAppendArray(bytes, int_bytes, sizeof(int_bytes));
+    VectorAppend(bytes, result.y);
 
     // Writes the witness type.
-    std::vector<unsigned char> witness_type = ConvertIntegerToBytes(integer(result.witness_type), 1);
-    bytes.insert(bytes.end(), witness_type.begin(), witness_type.end());
+    bytes.push_back(result.witness_type);
 
-    bytes.insert(bytes.end(), result.proof.begin(), result.proof.end());
+    VectorAppend(bytes, result.proof);
     std::string str_result = BytesToStr(bytes);
 
-    const uint32_t length = str_result.size();
-    std::vector<unsigned char> prefix_bytes = ConvertIntegerToBytes(integer(length), 4);
-    std::string prefix = BytesToStr(prefix_bytes);
+    Int32ToBytes(int_bytes, str_result.size());
 
     PrintInfo("Sending proof");
     {
         std::lock_guard<std::mutex> lock(socket_mutex);
-        boost::asio::write(sock, boost::asio::buffer(prefix_bytes, 4));
+        boost::asio::write(sock, boost::asio::buffer(int_bytes, 4));
         boost::asio::write(sock, boost::asio::buffer(str_result.c_str(), str_result.size()));
     }
     PrintInfo("Sent proof");
@@ -60,8 +60,8 @@ void CreateAndWriteProof(ProverManager& pm, uint64_t iteration, std::atomic<bool
     WriteProof(iteration, result, sock);
 }
 
-void CreateAndWriteProofOneWeso(uint64_t iters, integer& D, OneWesolowskiCallback* weso, std::atomic<bool>& stop_signal, tcp::socket& sock) {
-    Proof result = ProveOneWesolowski(iters, D, weso, stop_signal);
+void CreateAndWriteProofOneWeso(uint64_t iters, integer& D, form f, OneWesolowskiCallback* weso, std::atomic<bool>& stop_signal, tcp::socket& sock) {
+    Proof result = ProveOneWesolowski(iters, D, f, weso, stop_signal);
     if (stop_signal) {
         PrintInfo("Got stop signal before completing the proof!");
         return ;
@@ -78,7 +78,7 @@ void CreateAndWriteProofTwoWeso(integer& D, form f, uint64_t iters, TwoWesolowsk
     WriteProof(iters, result, sock);
 }
 
-char initial_form_s[100];
+uint8_t initial_form_s[BQFC_FORM_SIZE];
 
 void InitSession(tcp::socket& sock) {
     boost::system::error_code error;
@@ -151,7 +151,7 @@ void SessionFastAlgorithm(tcp::socket& sock) {
         integer D(disc);
         integer L=root(-D, 4);
         PrintInfo("Discriminant = " + to_string(D.impl));
-        form f = DeserializeForm(D, (uint8_t *)initial_form_s, sizeof(initial_form_s));
+        form f = DeserializeForm(D, initial_form_s, sizeof(initial_form_s));
         PrintInfo("Initial form: " + to_string(f.a.impl) + " " + to_string(f.b.impl));
         std::vector<std::thread> threads;
         const bool multi_proc_machine = (std::thread::hardware_concurrency() >= 16) ? true : false;
@@ -198,9 +198,8 @@ void SessionOneWeso(tcp::socket& sock) {
     try {
         integer D(disc);
         integer L=root(-D, 4);
-        form f=form::generator(D);
         PrintInfo("Discriminant = " + to_string(D.impl));
-
+        form f = DeserializeForm(D, initial_form_s, sizeof(initial_form_s));
         // Tell client that I'm ready to get the challenges.
         boost::asio::write(sock, boost::asio::buffer("OK", 2));
 
@@ -210,10 +209,10 @@ void SessionOneWeso(tcp::socket& sock) {
             return;
         }
         std::atomic<bool> stopped(false);
-        WesolowskiCallback* weso = new OneWesolowskiCallback(D, iter);
+        WesolowskiCallback* weso = new OneWesolowskiCallback(D, f, iter);
         FastStorage* fast_storage = NULL;
         std::thread vdf_worker(repeated_square, f, std::ref(D), std::ref(L), weso, fast_storage, std::ref(stopped));
-        std::thread th_prover(CreateAndWriteProofOneWeso, iter, std::ref(D), (OneWesolowskiCallback*)weso, std::ref(stopped), std::ref(sock));
+        std::thread th_prover(CreateAndWriteProofOneWeso, iter, std::ref(D), f, (OneWesolowskiCallback*)weso, std::ref(stopped), std::ref(sock));
         iter = ReadIteration(sock);
         while (iter != 0) {
             std::cout << "Warning: did not receive stop signal\n";
@@ -236,7 +235,7 @@ void SessionTwoWeso(tcp::socket& sock) {
         integer D(disc);
         integer L=root(-D, 4);
         PrintInfo("Discriminant = " + to_string(D.impl));
-        form f = DeserializeForm(D, (uint8_t *)initial_form_s, sizeof(initial_form_s));
+        form f = DeserializeForm(D, initial_form_s, sizeof(initial_form_s));
 
         // Tell client that I'm ready to get the challenges.
         boost::asio::write(sock, boost::asio::buffer("OK", 2));
