@@ -3,7 +3,7 @@
 
 #include "include.h"
 #include "integer_common.h"
-#include "vdf_new.h"
+#include "util.h"
 #include "picosha2.h"
 #include "nucomp.h"
 #include "proof_common.h"
@@ -11,6 +11,18 @@
 
 const uint8_t DEFAULT_ELEMENT[] = { 0x08 };
 
+int VerifyWesoSegment(integer &D, form x, form proof, integer &B, uint64_t iters, form &out_y)
+{
+    PulmarkReducer reducer;
+    integer L = root(-D, 4);
+    integer r = FastPow(2, iters, B);
+    form f1 = FastPowFormNucomp(proof, D, B, L, reducer);
+    form f2 = FastPowFormNucomp(x, D, r, L, reducer);
+    out_y = f1 * f2;
+
+    return B == GetB(D, x, out_y) ? 0 : -1;
+}
+
 void VerifyWesolowskiProof(integer &D, form x, form y, form proof, uint64_t iters, bool &is_valid)
 {
     PulmarkReducer reducer;
@@ -29,50 +41,31 @@ void VerifyWesolowskiProof(integer &D, form x, form y, form proof, uint64_t iter
     }
 }
 
-integer ConvertBytesToInt(const uint8_t* bytes, int32_t start_index, int32_t end_index)
-{
-    integer res(0);
-    bool negative = false;
-    if (bytes[start_index] & (1 << 7))
-        negative = true;
-    for (int32_t i = start_index; i < end_index; i++)
-    {
-        res = res * integer(256);
-        if (!negative)
-            res = res + integer(bytes[i]);
-        else
-            res = res + integer(bytes[i] ^ 255);
-    }
-    if (negative)
-    {
-        res = res + integer(1);
-        res = res * integer(-1);
-    }
-    return res;
-}
-
 bool CheckProofOfTimeNWesolowski(integer D, const uint8_t* x_s, const uint8_t* proof_blob, int32_t proof_blob_len, uint64_t iterations, uint64 disc_size_bits, int32_t depth)
 {
-    int form_size = bqfc_get_compr_size(D.num_bits());
+    int form_size = BQFC_FORM_SIZE;
+    int segment_len = 8 + B_bytes + form_size;
+    int i = proof_blob_len - segment_len;
     form x = DeserializeForm(D, x_s, form_size);
 
-    if (proof_blob_len != 2 * form_size + depth * (8 + 2 * form_size))
+    if (proof_blob_len != 2 * form_size + depth * segment_len)
         return false;
 
     // Loop depth times
     bool is_valid = false;
-    for (int32_t i = proof_blob_len - 2 * form_size - 8; i >= 2 * form_size; i -= 2 * form_size + 8)
-    {
-        auto iter_vector = ConvertBytesToInt(proof_blob, i, i + 8).to_vector();
-        form xnew = DeserializeForm(D, &proof_blob[i + 8], form_size);
-        VerifyWesolowskiProof(D, x,
-            xnew,
-            DeserializeForm(D, &proof_blob[i + 8 + form_size], form_size),
-            iter_vector[0], is_valid);
-        if(!is_valid)
+    for (; i >= 2 * form_size; i -= segment_len) {
+        uint64_t segment_iters = BytesToInt64(&proof_blob[i]);
+        form proof = DeserializeForm(D, &proof_blob[i + 8 + B_bytes], form_size);
+        integer B(&proof_blob[i + 8], B_bytes);
+        form xnew;
+        if (VerifyWesoSegment(D, x, proof, B, segment_iters, xnew))
             return false;
-        x=xnew;
-        iterations=iterations - iter_vector[0];
+
+        x = xnew;
+        if (segment_iters > iterations) {
+            return false;
+        }
+        iterations -= segment_iters;
     }
 
     VerifyWesolowskiProof(D, x,
