@@ -1,14 +1,15 @@
-from chiavdf import prove, verify_wesolowski, create_discriminant
 import secrets
 import time
 
+from chiavdf import create_discriminant, prove, verify_wesolowski
+
 
 def test_prove_and_verify():
     discriminant_challenge = secrets.token_bytes(10)
     discriminant_size = 512
     discriminant = create_discriminant(discriminant_challenge, discriminant_size)
-    form_size = discriminant_size // 32 * 3 + 4
-    initial_el = bytes([0x08])
+    form_size = 100
+    initial_el = b"\x08" + (b"\x00" * 99)
 
     iters = 1000000
     t1 = time.time()
